import pygame
from pygame.locals import *
import os
from lib import FileManager
from lib.ControlManager import ControlMapper
from lib.SpriteManager import SpriteManager
from lib.SimpleSurface import SimpleSurface
from lib.CollisionManager import CollisionManager
from lib.Shadows import *
from Test import Test
from Test2 import Test2

class Game:
    """Clase para heredar con ciertas funcionalidades para abstraer al programador de minijuegos"""
    def __init__(self, core):
        self.core = core
        self.controllableSprites = []
        self.itemsToRender = []
        self.core.players = 1 #1 por defecto
        self.core.gameMode = None #nada asignado por defecto
        self.started = False

    def startGame(self,name):
        """metodo que se inicializa al comenzar un minigame"""

        result = FileManager.xmlReader("minigames/"+name+"/info.xml")
            
        dicc = result[0]

        if dicc.has_key("info"):
            info = dicc["info"]

        if info.has_key("name"):
            title = info["name"]

        if info.has_key("description"):
            description = info["description"]

        players = self.core.players
        

        surface = pygame.Surface((self.core.screen_width * 0.8, self.core.screen_height * 0.8), SRCALPHA)
        surface.fill((255,255,255))
        
        surface = SimpleSurface(surface)

        nueva = add_shadow(surface.getSurface(), (2,2),1, 0.5)
        surface.setSurface(nueva)

        self.core.font.selectCustomFont("kimbalt_", 18)
        title = self.core.font.renderText(title, surface.max_rect().width * 0.5 , 10, (0,0,0), "center", "top")

        self.core.font.selectCustomFont("kimbalt_", 14)
        desc = self.core.font.renderParagraph(description, 0, 0, surface.max_rect().width * 0.99, surface.max_rect().height * 0.99)

        surface_to_blit = surface.getSurface()
        surface_to_blit.blit(title.getSurface(), (title.max_rect().left, title.max_rect().top))
        surface_to_blit.blit(desc.getSurface(), (10, title.max_rect().height + 20))
        surface.setSurface(surface_to_blit)

        if os.path.exists("minigames/"+name+"/instructions.png"):
            instructions = SimpleSurface("minigames/"+name+"/instructions.png")
            instructions.setRect(surface.max_rect().width * 0.5, surface.max_rect().height - surface.max_rect().height * 0.04, "center", "bottom")
            surface_to_blit.blit(instructions.getSurface(), (instructions.max_rect().left, instructions.max_rect().top))
        
        surface.setRect(self.core.screen_width * 0.1, self.core.screen_height * 0.1)
        self.startSurface = surface

        self.core.pause = True
            
            
    def paint(self):
        pass

    def logic(self):
        pass

    def bindSpriteToControls(self,sprite):
        self.controllableSprites.append(sprite)
        

    def controls(self):
        """rescatamos las funciones de mapeado general del juego y el resto lo delegamos al minijuego"""
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0 and self.core.pause == True:                            
            
            self.core.pause = False

            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))
            self.startSurface.hide()
            self.started = True

        if self.started == True:

            if self.core.controlManager.controlEvents.count(("KEYBUTTON", pygame.K_q)) > 0:
                self.core.controlManager.running = False
            
            if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_ESCAPE)) > 0:                            

                if self.core.gui.hasElement("genericbox-pause"):
                    self.core.gui.remove("genericbox-pause")
                    self.core.pause = False
                else:
                    self.core.gui.genericBox("pause", 100, 100, 500, 100)
                    self.core.pause = True

                self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_ESCAPE)))


            else:
                if self.core.pause == False:
                    controlMapper = ControlMapper(self.core.controlManager.controlEvents)
                    for sprite in self.controllableSprites:
                        sprite.controls(controlMapper)

    def paintItem(self, item):
        """agrega el sprite a la lista de elementos a pintar por el render"""
        if isinstance(item, dict):
            self.core.renderManager.paintGUI(item)
        else:
            if item.isPrintable():
                self.core.renderManager.paintItem(item)

    def setPlayers(self, num):
        """define el numero de jugadores que participaran"""
        self.core.players = num

    def setGameMode(self, mode):
        """define el modo de juego a ejecutar"""
        self.core.gameMode = mode

