import pygame
import os,sys
from lib import FileManager
from Game import Game
from game.intro.Fondo import Fondo
from lib.Shadows import *
from lib.SimpleSurface import SimpleSurface
from lib.GroupManager import GroupManager



class FreeMode(Game):
    """clase principal del menu del juego"""
    def __init__(self, core):
        Game.__init__(self,core)
        self.core = core
        self.fondo = Fondo()
        self.barra_superior = SimpleSurface("game/selectMode/images/barra_superior/barra_superior.png")

        self.core.font.selectCustomFont("kimbalt_", 30)

        self.fuente = self.core.font.renderText("Selecciona un minijuego:", self.core.screen_width * 0.01, self.core.screen_height * 0.05, (0,153,255))
        nueva = add_shadow(self.fuente.getSurface(), (2,2))
        self.fuente.setSurface(nueva)

        self.core.font.selectCustomFont("kimbalt_", 14)

        self.info = self.core.font.renderText("Usa Arriba - Abajo o Izquierda - Derecha para navegar, ESC para volver atras", self.core.screen_width - self.core.screen_width * 0.01, self.core.screen_height * 0.01, (255,0,0), "right", "top")
        nuevainfo = add_shadow(self.info.getSurface(), (1,1))
        self.info.setSurface(nuevainfo)

        self.selector = SimpleSurface("game/freeMode/images/select.png")
        
        
        self.selectedMinigame = 1
        self.initGridx = self.core.screen_width * 0.044
        self.initGridy = self.core.screen_height * 0.04
        self.gridPosx = 0
        self.gridPosy = 0
        self.max_objects_in_screen = 16
        self.offset_group = 0
        self.total_minigames = 0
        self.printGrid()
        
        self.core.font.selectCustomFont("kimbalt_", 18)

        self.minigamesCounter = self.core.font.renderText(str(self.total_minigames)+" minijuegos en total", self.core.screen_width - self.core.screen_width * 0.01, self.core.screen_height * 0.06, (0,0,0), "right", "top")
        #minigamesCounter_shadow = add_shadow(self.minigamesCounter.getSurface(), (1,1))
        #self.minigamesCounter.setSurface(minigamesCounter_shadow)
        
        
        
    def printGrid(self):
        """metodo que se usa para leer todos los minijuegos existentes y crear una rejilla para mostrarlos"""
        self.minigames=FileManager.dirReader("minigames",'d')
        self.group = GroupManager()
        self.total_minigames = len(self.minigames)
        #self.max_objects_in_screen * 0.5 porque en el grupo agregamos el doble de elementos: preview y namegame
        self.max_offset_group = int(self.total_minigames / (self.max_objects_in_screen * 0.5))
        
        i = 1

        for minigame in self.minigames:
            """leemos el archivo info.xml del minijuego"""
            if os.path.exists(FileManager.locationFixer("minigames/"+minigame+"/info.xml")):
                result = FileManager.xmlReader("minigames/"+minigame+"/info.xml")
            
                dicc = result[0]

                if dicc.has_key("info"):
                    
                    #imagen aclarativa del juego

                    self.preview = SimpleSurface("minigames/"+minigame+"/preview.png")
                    self.preview.setName(minigame)
                    self.preview.setSurface(pygame.transform.scale(self.preview.getSurface(), (200,140)))
                    self.preview.setRect(self.initGridx * (self.gridPosx + 1) + (self.preview.max_rect().width * self.gridPosx), (self.initGridy * (self.gridPosy + 1) + ((self.preview.max_rect().height + 50) * self.gridPosy)))

                    preview_shadow = add_shadow(self.preview.getSurface(), (4,4))
                    self.preview.setSurface(preview_shadow)

                    self.group.add(self.preview)
                        #informacion
                    self.core.font.selectSystemFont("Arial", 14)

                        
                    self.namegame = self.core.font.renderText(dicc["info"]["name"], self.preview.max_rect().left + self.preview.max_rect().width * 0.5, self.preview.max_rect().top + self.preview.max_rect().height + 10, (0,0,0), "center","top")

                    self.group.add(self.namegame)

                    self.gridPosx += 1
                    
                    if self.gridPosx > 3:
                        self.gridPosx = 0
                        self.gridPosy += 1
                    
                    i += 1

                    if i > self.max_objects_in_screen * 0.5:
                        self.gridPosy = 0
                        i = 1
                    
            else:
                #no ha encontrado el info
                print "error: no existe "+"minigames/"+minigame+"/info.xml"

        self.group.move(0, self.core.screen_height * 0.3)    

         
                  
                    
    def paint(self):
        self.paintItem(self.fondo)
        self.paintItem(self.barra_superior)
        self.paintItem(self.fuente)
        self.paintItem(self.info)

        self.paintItem(self.minigamesCounter)

        elements = self.group.subgroup(int(self.offset_group * self.max_objects_in_screen), self.max_objects_in_screen)

        for element in elements:
            """recorremos la lista para pintar todos los elementos"""
            if element == elements[(self.selectedMinigame - 1)*2]:
                #el minijuego esta seleccionado por lo que le aplicamos el efecto
                element_left = element.max_rect().left
                element_top = element.max_rect().top
                element_shadow = add_shadow(element.getSurface(), (0,0))
                element = SimpleSurface(element_shadow)
                element.setRect(element_left, element_top)
                self.selector.setRect(element_left + element.max_rect().width / 2, element_top + element.max_rect().height / 2, "center", "center")
                self.paintItem(element)
                self.paintItem(self.selector)
            else:
                self.paintItem(element)


       
        

    def logic(self):

        pass

    def controls(self):
        """rescatamos las funciones de mapeado general del juego y el resto lo delegamos al minijuego"""

        self.max_items_offset = (self.offset_group * int(self.max_objects_in_screen * 0.5)) + int(self.max_objects_in_screen * 0.5)

        if self.max_items_offset > self.total_minigames:
            self.max_items_offset = self.total_minigames - (self.max_items_offset - int(self.max_objects_in_screen * 0.5))
        
        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_DOWN)) > 0:
            self.selectedMinigame += 4
            if self.selectedMinigame > self.max_items_offset:
                #cambiamos a la siguiente tanda de elementos
                self.offset_group += 1
                if self.offset_group > self.max_offset_group:
                    self.offset_group = 0
                self.selectedMinigame = 1
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_DOWN)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RIGHT)) > 0:
            self.selectedMinigame += 1
            if self.selectedMinigame > self.max_items_offset:
                #cambiamos a la siguiente tanda de elementos
                self.offset_group += 1
                if self.offset_group > self.max_offset_group:
                    self.offset_group = 0
                self.selectedMinigame = 1
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RIGHT)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_UP)) > 0:
            self.selectedMinigame -= 4
            if self.selectedMinigame < 1:
                #cambiamos a la siguiente tanda de elementos
                self.offset_group -= 1
                if self.offset_group < 0:
                    self.offset_group = self.max_offset_group

                self.selectedMinigame = 1
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_UP)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_LEFT)) > 0:
            self.selectedMinigame -= 1
            if self.selectedMinigame < 1:
                #cambiamos a la siguiente tanda de elementos
                self.offset_group -= 1
                if self.offset_group < 0:
                    self.offset_group = self.max_offset_group

                self.selectedMinigame = 8
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_LEFT)))

        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_ESCAPE)) > 0:
            self.core.game = self.core.games[1]
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_ESCAPE)))


        if self.core.controlManager.controlSimple.count(("KEYBUTTON", pygame.K_RETURN)) > 0:
            search_minigames = self.group.subgroup(int(self.offset_group * self.max_objects_in_screen), self.max_objects_in_screen)
            
            
            minigame = __import__("minigames."+search_minigames[(self.selectedMinigame  - 1)* 2].getName()+".MiniGame", globals(), locals(),["MiniGame"], -1)
            
            self.core.game = minigame.MiniGame(self.core)
            
            self.core.controlManager.controlSimple.pop(self.core.controlManager.controlSimple.index(("KEYBUTTON", pygame.K_RETURN)))


        if self.core.controlManager.controlEvents.count(("KEYBUTTON", pygame.K_f)) > 0:
            self.core.toggleFullScreen()
     
        
