class AnimationManager:
    """Clase encargada de encapsular una secuencia de imagenes para hacer una animacion"""

    def __init__(self, sequence, name):
        """Inicializacion de parametros por defecto"""
        self.sequence = sequence
        self.name = name
        self.cursor = 0
        self.animationQueue = []
        self.printable = True
        self.pending_printable = True

    def getAnimation(self):
        """Devuelve la lista con la secuencia de imagenes"""
        return self.sequence

    def getName(self):
        """Devuelve el nombre de la animacion"""
        return self.name

    def getTotalImages(self, frames):
        """Cuenta numero total de frames de la animacion"""
        return len(frames)

    def addFrameToQueue(self):
        """Obtiene la imagen de la animacion a devolver y actualiza el cursor para devolver otra imagen en la siguiente llamada""" 
        current = self.cursor

        #comprobamos antes si el sprite puede imprimirse
        if current == 0 and self.pending_printable == False:
            self.printable = False

        self.cursor += 1

        if self.cursor >= self.getTotalImages(self.sequence):
            self.cursor = 0
        if len(self.animationQueue) == 0:
            self.animationQueue.append(self.sequence[current])

    def getFrame(self):
        if len(self.animationQueue) == 0: #si esta vacia la animacion devolvemos un frame por defecto
            return self.sequence[0]
        else:
            return self.animationQueue.pop(0)
        
