import pygame, sys, os
from pygame.locals import *
from ControlManager import ControlManager
from RenderManager import RenderManager
from SpriteManager import SpriteManager
from SoundManager import SoundManager
from GUIManager import GUIManager
from FontManager import *
from ConfigManager import ConfigHandler

class Engine(object):
    """Clase principal encargada de manejar todo el juego, gestionando la inicializacion de pygame, de los distintos modulos, variables y encargada de gestionar el loop principal del motor"""

    def __init__(self):
        """Constructor del motor de juego"""
        
        #iniciamos el motor de pygame
        pygame.init()

       

        self.setResolution()   
        #define titulo de la ventana del juego
        pygame.display.set_caption(self.title_game)

        #activa la visibilidad del raton en la pantalla
        pygame.mouse.set_visible(True)

        #activa la repeticion de botones
        #pygame.key.set_repeat(250,40)

        #Por defecto 60 ticks
        self.ticks = 60

        #contains updated sprites to render
        self.updatedSprites = []

        #Carga el modulo de configuracion
        self.configManager=ConfigHandler()


        #carga el modulo de controles
        self.controlManager = ControlManager()

        #carga el modulo de sonido
        self.soundManager = SoundManager()

        #carga el modulo de Sprites
        self.spriteManager = SpriteManager()

        #carga el modulo de fuentes
        self.font = FontManager()

        #carga el modulo de GUI
        self.gui = GUIManager()

        #carga el modulo de Renderizado
        self.renderManager = RenderManager(self)

        self.pause = False

        #lista donde se guardan los modulos a cargar a lo largo del juego
        self.games = []


    def setResolution(self):

        modes = pygame.display.list_modes()

        good_resolution = False

        while not good_resolution:

            mode = modes.pop(0)
           
            if mode <= (1024, 768):
                #Obtenemos la maxima resolucion de la pantalla para ajustes posteriores
                self.screen_width, self.screen_height = mode
                print mode,"\n"
                #Configuramos la pantalla principal con la resolucion y otras opciones
                self.screen = pygame.display.set_mode((self.screen_width,self.screen_height), pygame.FULLSCREEN)
                good_resolution = True

                if self.screen == None:
                    good_resolution = False

        
    def mainLoop(self):
        """Loop principal del juego, encargado de llamar a los modulos en cada iteraccion"""

        while self.controlManager.running:

            self.controlManager.capture()

            self.controls() #Este metodo debe implementarse en las clases de minijuegos
            
            self.logic() #Este metodo debe implementarse en las clases de minijuegos

            self.paint() #Este metodo debe implementarse en las clases de minijuegos

            self.renderManager.updateFrame()

            pygame.time.delay(self.ticks)


    def toggleFullScreen(self):
        pygame.display.toggle_fullscreen()

if __name__ == "__main__":
    print "this is a module and can't execute itself"

