import math


class Vector:

    def __init__(self,vect):
        """Vector representado como lista de dos o mas elementos."""
        self.vect=vect
    def getNorm(self):
        return math.sqrt(sum(map(lambda n: n**2,self.vect)))

    def isUnitVector(self):
        return (self.getNorm()==1)


class VectorOperator:

    def __init__(self):
        pass

    def sumVectors(self,vect1,vect2):
        newVect=[]
        lengthVect1=len(vect1.vect)
        lengthVect2=len(vect2.vect)
        for e in range(0,min(lengthVect1,lengthVect2)):
            newVect.append(vect1.vect[e]+vect2.vect[e])
        for e in range(min(lengthVect1,lengthVect2),max(lengthVect1,lengthVect2)):
            if lengthVect1>lengthVect2:
                newVect.append(vect1.vect[e])
            else:
                newVect.append(vect2.vect[e])
            
        return Vector(newVect)

    def reverseVector(self,vect1):
        return Vector(map(lambda n: n*-1,vect1.vect))



    def scalarProduct(self,*args):
        if (len(args)==2):
            return self.scalarProduct_Vect(*args)
        if (len(args)==3):
            return self.scalarProduct_Angle(*args)

    def scalarProduct_Vect(self,vect1,vect2):
        result=0
        lengthVect1=len(vect1.vect)
        lengthVect2=len(vect2.vect)
        if (lengthVect1==lengthVect2):
            for e in range(0,min(lengthVect1,lengthVect2)):
                result=result+vect1.vect[e]*vect2.vect[e]
        return result

    def scalarProduct_Angle(self,mod1,mod2,angle):
        radians=math.radians(angle)
        return float(mod1)*float(mod2)*math.cos(radians)


    def getAngle(self,vect1,vect2):
        scalarP=self.scalarProduct_Vect(vect1,vect2)
        mod1=vect1.getNorm()
        mod2=vect2.getNorm()
        if (mod1==0 or mod2==0):
            return 0
        result=scalarP/(mod1*mod2)
        return math.degrees(math.acos(result))

    def areOrtogonals(self,vect1,vect2):
        return (self.scalarProduct_Vect(vect1,vect2) == 0)

    def getUnitVector(self,vector):
        mod1=vector.getNorm()
        if (mod1==0):
            return None
        return Vector(map(lambda n: n/mod1,vector.vect))


    #TODO: Producto Vectorial



class Physic:

    def __init__(self,forcefield=0,gravity=40,ticks=60):
        self.gravity=float(gravity)        #Standard gravity on Y axis.
        self.forcefield=float(forcefield)  #Special gravity on X axis.
        self.ticks=ticks
        self.fpsGVector=Vector([(self.forcefield/self.ticks),(self.gravity/self.ticks)])
        self.vectorOps=VectorOperator()
        self.maxSpeed=math.sqrt((gravity*6)**2)

    def setGravity(self,ff,gr):
        self.gravity=gr
        self.forcefield=ff
        self.fpsGVector=Vector([ff/Engine.ticks,gr/Engine.ticks])

    def applyGravity(self,obj):
        obj.speedVect=self.vectorOps.sumVectors(obj.speedVect,self.fpsGVector)
        obj.move(int(obj.speedVect.vect[0]),int(obj.speedVect.vect[1]))

    def applyGravityAir(self,obj):
        if (obj.speedVect.getNorm()<self.maxSpeed):
            obj.speedVect=self.vectorOps.sumVectors(obj.speedVect,self.fpsGVector)
        obj.move(int(obj.speedVect.vect[0]),int(obj.speedVect.vect[1])) 
       
    def applyAcceleration(self,vect):
        obj.speedVect=self.vectorOps.sumVectors(obj.speedVect,obj.accelVect)
        obj.move(int(obj.speedVect.vect[0]),int(obj.speedVect.vect[1]))





#TODO: Betatesting
if __name__=="__main__":


    vectorop = VectorOperator()

    prueba1 = Vector([1, 2])
    prueba2 = Vector([1, 2])

    print vectorop.scalarProduct(prueba1,prueba2)
    print round(vectorop.scalarProduct(prueba1.getNorm(),prueba2.getNorm(),90),4)




