import pygame.mixer
import FileManager
import random

class SoundManager(object):
    """Clase que gestiona la musica y los sonidos del juego"""

    def __init__(self,freq=44100,size=16,stereo=2,buff=4096):
        pygame.mixer.pre_init(freq,size,stereo,buff)
        pygame.mixer.init()

        pygame.mixer.set_num_channels = 5
        pygame.mixer.set_reserved(2)

        self.reservedChannels=[pygame.mixer.Channel(0), pygame.mixer.Channel(1)]
        self.freeChannels=[pygame.mixer.Channel(2),pygame.mixer.Channel(3),pygame.mixer.Channel(4)]



    def playSound(self,location=''):
        sound = pygame.mixer.Sound(FileManager.locationFixer(location)) 
        try:
            sound.play()
        except pygame.error:
            print "Pygame.mixer.music -> File not found"
            return -1

    def playFreeSound(self,location='',left=0.5, right=0.5):
        chList=self.getFreeReady()
        if(chList<>[]):
            channel=chList[0]
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))
            channel.set_volume(left,right)
            try:
                channel.play(sound)
            except pygame.error:
                print "Pygame.mixer  Sound -> File not found"
                return -1

    def queueFreeSound(self,location=''):
        chList=self.getFreeReady()
        if(chList<>[]):
            channel=chList[0]
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))
            channel.queue(sound)
        else:
            channelNum=random.randint(0,len(self.freeChannels))
            channel=self.freeChannels[channelNum]
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))
            channel.queue(sound)


    def playReservedSound(self,location='',left=0.5, right=0.5):
        chList=self.getReservedReady()
        if(chList<>[]):
            channel=chList[0]
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))     
            channel.set_volume(left,right) 
            try:
                channel.play(sound)
            except pygame.error:
                print "Pygame.mixer  Sound -> File not found"
                return -1

    def queueReservedSound(self,location=''):
        chList=self.getReservedReady()
        if(chList<>[]):   
            channel=chList[0]   
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))
            channel.queue(sound)
        else:
            channelNum=random.randint(0,len(self.reservedChannels))
            channel=self.reservedChannels[channelNum]
            sound = pygame.mixer.Sound(FileManager.locationFixer(location))
            channel.queue(sound)

    def forcedPlaySound(self,location='',left=0.5, right=0.5):
                
        if self.areFreeBusy()==False:       
            self.playFreeSound(location,left,right)

        if self.areReservedBusy()==True:
            rand = random.randint(0, len(self.reservedChannels))
            sound = pygame.mixer.Sound(FileManager.locationFixer(location)) 
            self.reservedChannels[rand].play(sound)
            self.reservedChannels[rand].set_volume(left,right)
        else:
            self.playReservedSound(location,left,right)
        

    def forcedPlaySoundInChannel(self,channel, location='',left=0.5, right=0.5):        
        sound = pygame.mixer.Sound(FileManager.locationFixer(location)) 
        channel.set_volume(left, right)
        try:
            channel.play(sound)
        except pygame.error:
            print "Pygame.mixer  Sound -> File not found"
            return -1

    def quitMixer(self,):
        pygame.mixer.quit()


    def stereoXLocator(self, x, screenWidth):
        right = float(float(x) / screenWidth)
        left = float(1-right)
        sound = [left, right]
        return sound

    def areReservedBusy(self):
        busy=False
        for e in self.reservedChannels:
            busy = busy and e.get_busy()
        return busy

    def areFreeBusy(self):
        busy=False
        for e in self.freeChannels:
            busy = busy and e.get_busy()
        return busy
        
    def getReservedReady(self):
        chList=[]
        for e in self.reservedChannels:
            if not(e.get_busy()):
                chList.append(e)
        return chList

    def getFreeReady(self):
        chList=[]
        for e in self.freeChannels:
            if not(e.get_busy()):
                chList.append(e)
        return chList

    #-------MUSIC----------


    def playMusic(self,location='',times=-1):
        try:
            pygame.mixer.music.load(FileManager.locationFixer(location))
            pygame.mixer.music.play(times)
        except pygame.error:
            print "Pygame.mixer.music -> File not found"
            return -1

    def fadeoutMusic(self,milisec=2000):
        pygame.mixer.music.fadeout(int(milisec))

    def isMusicBusy(self):
        return pygame.mixer.music.get_busy()

    def pauseMusic(self):
        pygame.mixer.music.pause()

    def resumeMusic(self):
        pygame.mixer.music.unpause()
    def unpauseMusic(self):
        pygame.mixer.music.unpause()

    def stopMusic(self):
        pygame.mixer.music.stop()

    def rewindMusic(self):
        pygame.mixer.music.rewind()


if __name__=="__main__":
    

    so=SoundManager()
    print so.stereoXLocator(300,640)
    #so.playFreeSound("bocadillo.wav",1.0,0) 
    print so.getReservedReady()
    print so.getFreeReady()
    

    eleccion = raw_input("Ruta del archivo:")
    so.playMusic(eleccion)


    nombre=None
    while nombre<>"Salir" and nombre<>"salir":

        if nombre == "fadeout":
            so.fadeoutMusic()
        elif nombre == "isbusy":
            print so.isMusicBusy()
        elif nombre == "stop":
            print so.stopMusic()
        elif nombre == "pause":
            so.pauseMusic()
        elif nombre == "resume":
            so.resumeMusic()
        elif nombre == "unpause":
            so.unpauseMusic()
        elif nombre == "load":
            ruta = raw_input("Ruta del archivo:")
            so.playMusic(ruta)
            
        else:
            pass


        nombre = raw_input("Orden:")




