import pygame
import FileManager
from AnimationManager import AnimationManager
import sys, os
import math
from PhysicsManager import Vector

class SpriteManager(pygame.sprite.Sprite):
    """Clase que implementa funciones para la generacion de Sprites dotados de animacion y control de colisiones"""
    
    def __init__(self, surface = None):
        """Inicializacion del objeto Sprite llamando al constructor padre de DirtySprite y definiendo las variables por defecto"""
        self.surface = surface
        pygame.sprite.Sprite.__init__(self)
        self.sequence = []
        self.left = 0
        self.top = 0
        self.key_pressed = []
        self.rect = None
        self.speedVect=Vector([0,0])
        self.accelVect=Vector([0,0])

    def load_sequence(self, directory, mirror = False):
        """Obtiene las imagenes y las carga en memoria para su posterior uso"""
        print directory
        sequence = []
        fileList = FileManager.dirReader(directory, 'f')
        fileList.sort()

        #TODO: PRUEBAS PYTOEXE
        print fileList

        for image in fileList:
            if mirror == False:
                #sequence.append(pygame.image.load(directory + os.path.sep + image).convert_alpha())
                sequence.append(pygame.image.load(FileManager.locationFixer(directory + os.path.sep + image)).convert_alpha())
            else:
                #sequence.append(pygame.transform.flip(pygame.image.load(directory+os.path.sep+image).convert_alpha(), True, False))         
                sequence.append(pygame.transform.flip(pygame.image.load(FileManager.locationFixer(directory+os.path.sep+image)).convert_alpha(), True, False))    

        return sequence

    def isPrintable(self):
        return self.animation.printable
    
    def hide(self):
        self.animation.pending_printable = False

    def show(self):
        self.animation.printable = True

    def getAnimation(self, directory, name, mirror = False):
   
        self.sequence = self.load_sequence(directory, mirror)

        self.animation = AnimationManager(self.sequence, name)
        return self.animation

    def update():
        pass

    def max_rect(self):
        """Obtiene el tamanio maximo de todos los fotogramas para repintar"""
        
        max_rect_width = 0
        max_rect_height = 0
        
        for frame in self.sequence:
            if frame.get_width() > max_rect_width:
                max_rect_width = frame.get_width()

            if frame.get_height() > max_rect_height:
                max_rect_height = frame.get_height()

        return pygame.Rect(self.rect.left, self.rect.top, max_rect_width, max_rect_height)

    def animate(self, animation):
        """Metodo que devuelve el frame concreto para cada iteraccion del loop principal"""
        #return animation.getFrame()
        if self.isPrintable():
            self.animation = animation
            self.animation.addFrameToQueue()
    
    def getSurface(self):
        """Devuelve la superficie a pintar por el render"""
        self.surface = self.animation.getFrame()
        self.image = self.surface
        return self.surface

    def get_rect(self):
        return self.rect

    def defaultSurface(self, default):
        """Crea una superficie por defecto al inicializar el sprite"""
        self.surface = self.animation.getFrame()
        self.image = self.surface
        self.rect = self.surface.get_rect()
        self.rect.width = self.max_rect().width
        self.rect.height = self.max_rect().width
        self.old_rect = self.rect

    def loadSprites(self):
        """Metodo a implementar en la clase que hereda de SpriteManager para cargar los objetos con las secuencias de imagenes"""
        pass

    def initialPosition(self, left, top, absolute = "left"):
        """Metodo que define la posicion inicial del Sprite"""
        try:
            if absolute == "left":
                self.rect.left = left
                self.rect.top = top
            elif absolute == "center":
                self.rect.left = left - self.rect.width/2
                self.rect.top = top - self.rect.height/2
        except:
            print "Debes llamar antes al metodo defaultSurface"

    def move(self,x,y):
        """Metodo para mover el sprite por la pantalla"""
        if self.isPrintable():
            self.old_rect = self.rect
            self.rect = self.rect.move(x,y)

    def angleMove(self,r,angle):
        
        if self.isPrintable():
            anglerad = (angle * math.pi)/180
            dx = r* math.cos(anglerad)
            dy = r* math.sin(anglerad)
            self.move(dx,dy)


